const { pool } = require("../config/database/database.config");

/*
Function to add a row into the peopleContact
table with the contact info given. Returns the id of the new contact.
If no contact is added, null is returned
*/
async function addContact(contact, peopleId) {
  // extract the parts of the contact
  let { contactNumber, contactType, preferredContact } = contact;

  console.log("adding contact....");

  try {
    // query for adding the contact
    const query = `
        INSERT INTO peopleContact (peopleId, contactNumber, contactType, preferredContact)
        VALUES (?,?,?,?)
    `;

    // insert data into db
    const [results] = await pool.query(query, [
      peopleId,
      contactNumber,
      contactType,
      preferredContact,
    ]);

    return results.insertId;
  } catch (error) {
    console.log(error);
    return null;
  }
}

/*
Function to update the data for a specific contact. The 
function will return true if the update was successful and false
otherwise.
*/
async function updateContact(peopleContactId, peopleId, contactInfo) {
  // extract the parts of the contact
  let { contactNumber, contactType, preferredContact } = contactInfo;

  console.log("updating contact....");

  try {
    const query = `
        UPDATE peopleContact
        SET
          peopleId = ?,
          contactNumber = ?,
          contactType = ?,
          preferredContact = ?
        WHERE peopleContactId = ?
      `;

    const [results] = await pool.query(query, [
      peopleId,
      contactNumber,
      contactType,
      preferredContact,
      peopleContactId,
    ]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Contact not found. no rows updated");
      return false;
    }

    // return true to signify success
    return true;
  } catch (error) {
    console.log("Error updating contact: ", error);
    return false;
  }
}

/*
Function to find all contact information associated with a person.
Returns an array of contact objects or an empty array if no contacts exist.
*/
async function getContactsForPerson(peopleId) {
  console.log("Finding contacts for person with ID: ", peopleId);

  try {
    // get all contacts with the wanted peopleId
    const query = `
      SELECT 
        peopleContactId, 
        contactNumber, 
        contactType, 
        preferredContact
      FROM 
        peopleContact
      WHERE 
        peopleId = ?;
    `;

    // Execute the query
    const [results] = await pool.query(query, [peopleId]);

    if (results.length === 0) {
      console.log("No contacts found for person with ID:", peopleId);
      return []; // empty array, no contacts found
    }

    // Map the results to structured contact objects
    const contacts = results.map((row) => ({
      peopleContactId: row.peopleContactId,
      contactNumber: row.contactNumber,
      contactType: row.contactType,
      preferredContact: row.preferredContact,
    }));

    console.log("Found contacts: ", contacts);

    return contacts;
  } catch (error) {
    console.error("Error fetching contacts for person: ", error);
    return []; // empty array, no contacts found
  }
}

/*
Function to delete a contact by its ID. 
Returns true if the deletion was successful and false if no rows were affected.
*/
async function deleteContactById(peopleContactId) {
  console.log("Deleting contact with ID: ", peopleContactId);

  try {
    const query = `
      DELETE FROM peopleContact
      WHERE peopleContactId = ?
    `;

    const [results] = await pool.query(query, [peopleContactId]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Contact not found. No rows deleted.");
      return false;
    }

    console.log("Contact deleted successfully.");
    return true;
  } catch (error) {
    console.error("Error deleting contact: ", error);
    return false;
  }
}

module.exports = {
  addContact,
  updateContact,
  getContactsForPerson,
  deleteContactById,
};
